class Blob:
    type = "blob"

    def __init__(self, null=False, comment=None):
        self.length = None
        self.default = None
        self.null = null
        self.primary = False
        self.comment = comment
        self.auto = False
        self._value = None

    def __len__(self):
        return self.length

    def __set__(self, instance, value):
        if not self.null and value is None:
            raise ValueError("不能为空!")
        elif not isinstance(value, str):
            raise TypeError(f"{value}必须是字符串!")
        else:
            self._value = value

    def __get__(self, instance, owner):
        return self._value

    def __delete__(self, instance):
        del self._value



from typing import Sequence
from pydb3.config.property import *
from pydb3.config.weight import *
from pydb3.utils.valid import table_valid, data_valid


class TableGenerator:
    db_type = None

    def __init__(self):
        self.op_dict = dict()

    @data_valid("create")
    @table_valid
    def create(self, db_type='sqlite', table=None, fields: Sequence = None):
        self.db_type = db_type
        self.op_dict[OP_WEIGHT] = "CREATE TABLE"
        self.op_dict[CREATE_TABLE_WEIGHT] = f"`{table}`"
        for field in fields:
            self.create_field(field)
        return self.sql

    @classmethod
    def type_map(cls, db_type, field):
        return DB_FIELD_MAP.get(db_type).get(field)

    def create_field(self, field: FieldProperty):
        field_type = self.type_map(self.db_type, field.type)
        field_length = f'({field.length})' if not field.auto and field.length else ''
        self.op_dict[CREATE_TABLE_FIELD_WEIGHT] = [field.name]
        self.op_dict[CREATE_TABLE_FIELD_WEIGHT].append(f"{field_type}{field_length}")
        self.op_dict[CREATE_TABLE_FIELD_WEIGHT].append('NULL' if field.null else 'NOT NULL')
        self.op_dict[CREATE_TABLE_FIELD_WEIGHT].append('PRIMARY KEY' if field.primary else '')
        self.op_dict[CREATE_TABLE_FIELD_WEIGHT].append('autoincrement' if field.auto else '')
        self.op_dict[CREATE_TABLE_FIELD_WEIGHT].append(f',-- {field.comment}' if field.comment else ',')
        self.op_dict[CREATE_TABLE_FIELD_WEIGHT] = " ".join(self.op_dict[CREATE_TABLE_FIELD_WEIGHT])
        if self.op_dict.get(CREATE_TABLE_FIELDS_WEIGHT):
            self.op_dict[CREATE_TABLE_FIELDS_WEIGHT].append(self.op_dict[CREATE_TABLE_FIELD_WEIGHT])
        else:
            self.op_dict[CREATE_TABLE_FIELDS_WEIGHT] = [self.op_dict[CREATE_TABLE_FIELD_WEIGHT]]

    @property
    def sql(self):
        self.op_dict[CREATE_TABLE_FIELDS_WEIGHT][-1] = self.op_dict[CREATE_TABLE_FIELDS_WEIGHT][-1].replace(',', '')
        self.op_dict[CREATE_TABLE_FIELDS_WEIGHT] = "\n\t".join(self.op_dict[CREATE_TABLE_FIELDS_WEIGHT])
        result = f"{self.op_dict[OP_WEIGHT]} {self.op_dict[CREATE_TABLE_WEIGHT]} (\n\t{self.op_dict[CREATE_TABLE_FIELDS_WEIGHT]}\n); "
        return result



from pydb3.generator.public import PublicGenerator
from pydb3.config.weight import *
from pydb3.utils.valid import table_valid


class DeleteGenerator(PublicGenerator):
    def __init__(self) -> None:
        super().__init__()

    @table_valid
    def delete(self, table):
        self.op_dict[OP_WEIGHT] = "DELETE"
        self.op_dict[DELETE_FROM_WEIGHT] = "FROM"
        self.op_dict[DELETE_TABLE_WEIGHT] = f"`{table}`"
        return self

    def execute(self):
        raise NotImplemented

    def __repr__(self):
        return '删除操作'


if __name__ == "__main__":
    deleter = DeleteGenerator()
    deleter.delete("test").where(a=1, b=2).and_()
    print(deleter.sql)



import os
from functools import wraps


def log(name, command=False, file=True, level=0):
    def wrapper(func):
        @wraps(func)
        def inner(*args, **kwargs):
            write_path = f"{os.getcwd()}/{name}.log"
            db_card = func(*args, **kwargs)
            if command and (level or level == db_card.status):
                print(db_card.log)
            if file:
                with open(write_path, "a", encoding="utf-8") as fp:
                    fp.write(db_card.log + "\n")
            return db_card
        return inner

    return wrapper



from collections import namedtuple

DBCard = namedtuple("DBCard", "results log status")
FieldProperty = namedtuple("FieldProperty", "name type length auto default null primary comment")

SQLITE_TYPE_MAP = {
    "char": "char",
    "integer": "integer",
    "float": "float",
    "double": "double",
    "text": "text",
    "varchar": "varchar",
    "date": "date",
    "datetime": "datetime",
    'blob': 'blob',
    'image': 'blob'
}

DB_FIELD_MAP = {
    'sqlite': SQLITE_TYPE_MAP
}



from functools import wraps


def table_valid(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        if args[-1] is None:
            raise ValueError("表名不能为空!")
        return func(*args, **kwargs)

    return wrapper


def data_valid(op):
    def inner(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            if op == 'insert' and not args[-1]:
                raise ValueError("无插入数据!")
            elif op == "update" and not kwargs:
                raise ValueError("无更新字段!")
            elif op == "create" and not args[-1]:
                raise ValueError("无创建字段!")
            elif op == 'join' and len(args) < 3:
                raise ValueError("无连接字段!")
            return func(*args, **kwargs)

        return wrapper

    return inner



class Integer:
    _value = 0
    type = "integer"

    def __init__(self, length=11, default=None, null=False, auto=False, primary=False, comment=None):
        self.length = length
        self.default = default
        self.null = null
        self.primary = primary
        self.auto = auto
        self.comment = comment

    def __len__(self):
        return self.length

    def __set__(self, instance, value):
        if not value:
            self._value = self.default if not self.auto else None
            if not self.null and not self._value and not self.auto:
                raise ValueError("不能为空!")
        else:
            if not isinstance(value, int):
                raise TypeError(f"{value}必须是整数!")
            elif len(str(value)) > self.length:
                raise ValueError("整形超出长度!")
            else:
                self._value = value

    def __get__(self, instance, owner):
        return self._value

    def __delete__(self, instance):
        del self._value



from typing import Sequence
from pydb3.config.weight import *
from pydb3.generator.public import Filter
from pydb3.utils.valid import table_valid, data_valid


class InsertGenerator:
    def __init__(self) -> None:
        self.op_dict = dict()
        self._sql = None

    @table_valid
    @data_valid('insert')
    def insert(self, table, fields: Sequence):
        self.op_dict[OP_WEIGHT] = 'INSERT INTO'
        self.op_dict[INSERT_TABLE_WEIGHT] = f"`{table}`"
        fields = [Filter.map(field, True) for field in fields]
        values = ",".join(fields)
        self.op_dict[INSERT_VALUES_WEIGHT] = f"VALUES({values})"
        return self

    @property
    def sql(self):
        if self._sql:
            return self._sql
        else:
            op_dict = sorted(self.op_dict.items(), key=lambda a: a[0])
            self._sql = " ".join([op[1] for op in op_dict])
            return self._sql

    def execute(self):
        raise NotImplemented

    def __repr__(self):
        return '插入操作'


if __name__ == "__main__":
    inserter = InsertGenerator()
    inserter.insert("test", [])



from pydb3.config.weight import *
from pydb3.generator.public import PublicGenerator, Filter
from pydb3.utils.valid import table_valid, data_valid


class UpdateGenerator(PublicGenerator):
    def __init__(self) -> None:
        super().__init__()

    @table_valid
    @data_valid("update")
    def update(self, table, **kwargs):
        if not kwargs:
            raise ValueError("更新参数为空!")
        self.op_dict[OP_WEIGHT] = 'UPDATE'
        self.op_dict[UPDATE_TABLE_WEIGHT] = f"`{table}`"
        self.op_dict[UPDATE_SET_WEIGHT] = "SET"
        self.op_dict[UPDATE_SET_FIELD_WEIGHT] = ",".join(
            [f"`{key}` = {Filter.map(value)}" for key, value in kwargs.items()])
        return self

    def execute(self):
        raise NotImplemented

    def __repr__(self):
        return '更新操作'


if __name__ == "__main__":
    updater = UpdateGenerator()
    updater.update("student", name=None, age=12, gender='男').where(name=12, age=13) | updater.where(
        a=123) | updater.like(b=1234)
    print(updater.sql)



from pydb3.model.char import Char
from pydb3.model.integer import Integer
from pydb3.model.float import Float
from pydb3.model.double import Double
from pydb3.model.varchar import VarChar
from pydb3.model.text import Text
from pydb3.model.date import Date
from pydb3.model.datetime import DateTime
from pydb3.model.blob import Blob
from pydb3.model.image import Image
from pydb3.model.model import Model



from pydb3.config.weight import *


class PublicGenerator:
    def __init__(self):
        self.op_dict = dict()
        self._sql = None

    def and_(self, equal_op='=', **kwargs):
        return self.where(equal_op, 'AND', **kwargs)

    def or_(self, equal_op='=', **kwargs):
        return self.where(equal_op, 'OR', **kwargs)

    def where(self, equal_op='=', condition='AND', is_group=False, **kwargs):
        condition_op_weight = WHERE_WEIGHT if not is_group else SELECT_HAVING_WEIGHT
        condition_op = 'WHERE' if not is_group else 'HAVING'
        condition_weight = CONDITION_WEIGHT if not is_group else SELECT_HAVING_CONDITION
        if kwargs:
            self.op_dict[condition_op_weight] = condition_op
            for key, value in kwargs.items():
                value = Filter.map(value)
                condition_str = f"`{key}` {equal_op} {value}"
                if not self.op_dict.get(condition_weight):
                    self.op_dict[condition_weight] = [condition_str]
                else:
                    self.op_dict[condition_weight].append(condition)
                    self.op_dict[condition_weight].append(condition_str)
        return self

    def like(self, **kwargs):
        return self.where('LIKE', **kwargs)

    @property
    def sql(self):
        if self._sql:
            return self._sql
        else:
            if self.op_dict.get(CONDITION_WEIGHT):
                self.op_dict[CONDITION_WEIGHT] = " ".join(self.op_dict[CONDITION_WEIGHT])

            if self.op_dict.get(SELECT_HAVING_CONDITION):
                self.op_dict[SELECT_HAVING_CONDITION] = " ".join(self.op_dict[SELECT_HAVING_CONDITION])

            op_dict = sorted(self.op_dict.items(), key=lambda a: a[0])
            self._sql = " ".join([op[1] for op in op_dict])
            return self._sql


class Filter:
    def __init__(self):
        pass

    @classmethod
    def map(cls, value, insert=False):
        if value is None:
            return "NULL"
        elif isinstance(value, str) or insert:
            return f"'{value}'"
        else:
            return value



from datetime import date


class Date:
    type = "date"

    def __init__(self, length=11, default=None, null=False, auto=True, primary=False, comment=None):
        self.length = length
        self.default = default
        self.null = null
        self.primary = primary
        self.comment = comment
        self.auto = auto
        self._value = date.today()

    def __len__(self):
        return self.length

    def __set__(self, instance, value):
        if not value and self.auto:
            self._value = date.today()
        elif isinstance(value, str):
            value = value.replace(" ", "-").replace(",", "-")
            dates = value.split("-")
            if len(dates) < 3:
                raise ValueError("日期不正确!")
            year, month, day = map(int, dates)
            self._value = date(year, month, day)
        elif isinstance(value, date):
            self._value = date
        elif value:
            raise TypeError("日期类型不正确!")

    def __get__(self, instance, owner):
        return self._value

    def __delete__(self, instance):
        del self._value



import sqlite3
from typing import Sequence
from functools import wraps
from pydb3.config.property import DBCard
from pydb3.generator.select import SelectGenerator
from pydb3.generator.update import UpdateGenerator
from pydb3.generator.delete import DeleteGenerator
from pydb3.generator.insert import InsertGenerator
from pydb3.log.log import log


def sqlite_execute(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        file_path, sql, selector = func(*args, **kwargs)
        conn = sqlite3.connect(file_path)
        sql = sql or selector.sql
        msg = f"{selector}成功!--{sql}"
        status = 1
        cursor = conn.cursor()
        try:
            cursor.execute(sql)
            conn.commit()
        except Exception as e:
            msg = f"{e}--{sql}"
            status = 0
        db_card = DBCard(results=cursor.fetchall(), log=msg, status=status)
        return db_card

    return wrapper


class Sqlite:
    def __init__(self, file_path="test.db") -> None:
        self.selector = SelectGenerator()
        self.updater = UpdateGenerator()
        self.deleter = DeleteGenerator()
        self.inserter = InsertGenerator()
        self.file_path = file_path
        self.operator = None

    def select(self, table=None, *fields) -> SelectGenerator:
        self.operator = self.selector
        self.operator.execute = self.execute
        return self.operator.select(table, *fields)

    def update(self, table, **kwargs) -> UpdateGenerator:
        self.operator = self.updater
        self.operator.execute = self.execute
        return self.operator.update(table, **kwargs)

    def insert(self, table, fields: Sequence) -> InsertGenerator:
        self.operator = self.inserter
        self.operator.execute = self.execute
        return self.operator.insert(table, fields)

    def delete(self, table) -> DeleteGenerator:
        self.operator = self.deleter
        self.operator.execute = self.execute
        return self.operator.delete(table)

    @log(name="sqlite", command=True, file=True)
    @sqlite_execute
    def execute(self, sql=None):
        return self.file_path, sql, self.operator


if __name__ == '__main__':
    sqlite = Sqlite(r"E:\python\db-library\data\test.db")
    a = sqlite.update("student", name="test").where(name="李白").like(age='%2%').execute()
    print(a)



from _datetime import datetime
import time


class DateTime:
    type = "datetime"

    def __init__(self, length=20, default=None, null=False, auto=True, primary=False, comment=None):
        self.length = length
        self.default = default
        self.null = null
        self.primary = primary
        self.comment = comment
        self.auto = auto
        self._value = datetime.now().strftime("%Y-%m-%d %H:%M:%S")

    def __len__(self):
        return self.length

    def __set__(self, instance, value):
        if not value and self.auto:
            self._value = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        elif isinstance(value, str):
            value = time.strftime(value)
            self._value = value
        elif isinstance(value, datetime):
            self._value = value
        elif value:
            raise TypeError("时间类型不正确!")

    def __get__(self, instance, owner):
        return self._value

    def __delete__(self, instance):
        del self._value


if __name__ == '__main__':
    print(len('2022-08-13 00:00:00'))



from typing import Sequence

from pydb3.generator.select import SelectGenerator
from pydb3.generator.update import UpdateGenerator
from pydb3.generator.delete import DeleteGenerator
from pydb3.generator.insert import InsertGenerator
from pydb3.model import Model


class Db:

    def __init__(self, db_type="sqlite", *args, **kwargs) -> None:
        db_class = self.from_db(db_type)
        self.db = db_class(*args, **kwargs)
        self.db_type = db_type

    @classmethod
    def from_db(cls, db_type='sqlite'):
        class_name = db_type.capitalize()
        db_module = __import__(f"pydb3.adapter.{db_type}", fromlist=[class_name])
        db_class = getattr(db_module, class_name)
        return db_class

    def execute(self, sql):
        return self.db.execute(sql)

    def select(self, table=None, *fields) -> SelectGenerator:
        return self.db.select(table, *fields)

    def update(self, table, **kwargs) -> UpdateGenerator:
        return self.db.update(table, **kwargs)

    def delete(self, table) -> DeleteGenerator:
        return self.db.delete(table)

    def insert(self, table, fields) -> InsertGenerator:
        return self.db.insert(table, fields)

    def create_table(self, model: Model):
        model.set_db(self)
        model.create_table()

    @property
    def sql(self):
        return self.db.sql


if __name__ == '__main__':
    pass



import base64

from pydb3.model import Blob
from pydb3.utils.request import download


def save_image(image, image_path):
    image = str(image).encode("utf-8")
    image = base64.b64decode(image)
    with open(image_path, 'wb') as fp:
        fp.write(image)


class Image(Blob):
    type = 'image'

    def __set__(self, instance, value):
        if "path=" in value:
            value = value.split('path=')[-1]
            save_image(self._value, value)
        elif "url=" in value:
            url = value.replace('url=', '')
            self._value = base64.b64encode(download(url)).decode('utf-8')
        else:
            if not self.null and value is None:
                raise ValueError("不能为空!")
            elif not isinstance(value, str):
                raise TypeError(f"{value}必须是字符串!")
            else:
                self._value = value



from pydb3.config.property import FieldProperty
from pydb3.generator.table import TableGenerator

other = ("__module__", "__main__", "__init__", "__doc__", "db", 'table')


class Model:
    db = None
    table = None

    @classmethod
    def set_db(cls, db):
        cls.db = db

    @classmethod
    def add_underline(cls, class_name) -> str:
        count = 0
        ch_list = list()
        for ch in class_name:
            if 'A' <= ch <= 'Z':
                count += 1
                if count > 1:
                    ch_list.append("_")
            ch_list.append(ch)
        class_name = "".join(ch_list)
        return class_name.lower()

    @classmethod
    def create_table(cls):
        table_generator = TableGenerator()
        table_name = cls.add_underline(cls.__name__)
        fields = list()
        for key, value in cls.__dict__.items():
            if key not in other:
                field_length = getattr(value, "length")
                field_type = getattr(value, "type")
                field_default = getattr(value, "default")
                field_auto = getattr(value, "auto") if hasattr(value, "auto") else False
                field_primary = getattr(value, "primary")
                field_null = getattr(value, "null")
                field_comment = getattr(value, 'comment')
                field_property = FieldProperty(
                    name=key, type=field_type, length=field_length,
                    default=field_default, auto=field_auto,
                    primary=field_primary, null=field_null,
                    comment=field_comment
                )
                fields.append(field_property)
        sql = table_generator.create(cls.db.db_type, table_name, fields)
        print(sql)
        cls.db.execute(sql)

    def save(self):
        self.table = self.add_underline(self.__class__.__name__)
        values = list()
        for key, value in self.__class__.__dict__.items():
            if key not in other:
                value = getattr(value, '_value')
                values.append(value)
        self.db.insert(self.table, values).execute()

    @classmethod
    def find(cls, **kwargs) -> list:
        cls.table = cls.add_underline(cls.__name__)
        items = list()
        results = cls.db.select(cls.table).where(**kwargs).execute().results
        fields = tuple(filter(lambda x: x not in other, cls.__dict__.keys()))
        for result in results:
            obj = super().__new__(cls, **kwargs)
            for i in range(len(result)):
                try:
                    setattr(obj, fields[i], result[i])
                except Exception as e:
                    raise ValueError('属性设置出错!', e)
            items.append(obj)
        return items

    def __repr__(self):
        fields = list()
        for key, value in self.__class__.__dict__.items():
            if key not in other:
                value = getattr(value, '_value')
                fields.append(f"{key}={value}")
        return f"({','.join(fields)})"



from pydb3.config.property import *
from pydb3.config.weight import *
from pydb3.generator.public import PublicGenerator
from pydb3.utils.valid import table_valid, data_valid


class SelectGenerator(PublicGenerator):
    def __init__(self) -> None:
        super().__init__()
        self.table = None

    def __getitem__(self, s: slice):
        size = s.stop - s.start
        return self.limit(s.start, size)

    @table_valid
    def select(self, table=None, *fields):
        self._sql = None
        self.table = table
        self.op_dict = dict()
        self.op_dict[OP_WEIGHT] = 'SELECT'
        self.op_dict[SELECT_TABLE_WEIGHT] = f"`{table}`"
        select_fields = "*"
        if len(fields) > 0:
            select_fields = ",".join(fields)
        self.op_dict[SELECT_FIELD_WEIGHT] = select_fields
        self.op_dict[SELECT_FROM_WEIGHT] = 'FROM'
        return self

    def distinct(self, *field):
        op = 'DISTINCT'
        select_fields = self.op_dict[SELECT_FIELD_WEIGHT]
        for f in field:
            if f'{op} {f}' not in select_fields:
                select_fields = select_fields.replace(f, f'{op} {f}')
        self.op_dict[SELECT_FIELD_WEIGHT] = select_fields
        return self

    def execute(self) -> DBCard:
        raise NotImplemented

    def limit(self, begin=0, size=0):
        self.op_dict[SELECT_LIMIT_WEIGHT] = "LIMIT"
        self.op_dict[SELECT_LIMIT_VALUE_WEIGHT] = f"{begin},{size}"
        return self

    def order_by(self, fields, desc=False):
        try:
            fields = tuple(fields.replace(' ', '').replace(",", " ").split())
        except:
            fields = tuple(fields)
        self.op_dict[SELECT_ORDER_WEIGHT] = 'ORDER BY'
        self.op_dict[SELECT_ORDER_FIELD_WEIGHT] = ','.join(fields)
        self.op_dict[SELECT_ORDER_ASC_WEIGHT] = 'ASC' if not desc else 'DESC'
        return self

    def group_by(self, fields):
        try:
            fields = tuple(fields.replace(' ', '').replace(",", " ").split())
        except:
            fields = tuple(fields)
        self.op_dict[SELECT_GROUP_WEIGHT] = 'GROUP BY'
        self.op_dict[SELECT_GROUP_FIELD] = ','.join(fields)
        return self

    def having(self, equal_op='=', condition='AND', **kwargs):
        return self.where(equal_op, condition, True, **kwargs)

    def join(self, table=None, *fields):
        self.op_dict[SELECT_JOIN_WEIGHT] = 'JOIN'
        self.op_dict[SELECT_JOINED_TABLE_WEIGHT] = f'`{table}` ON'
        self.op_dict[SELECT_JOIN_CONDITION_WEIGHT] = ' AND '.join(
            f"{self.table}.{field}={table}.{field}" for field in fields)
        return self

    def count(self):
        field = self.op_dict[SELECT_FIELD_WEIGHT].split(',')[0]
        self.op_dict[SELECT_FIELD_WEIGHT] = f'Count({field})'
        return self

    def __repr__(self):
        return '查询操作'


if __name__ == "__main__":
    selector = SelectGenerator()
    sql = selector.select('table').count().sql
    print(sql)



OP_WEIGHT = 0
WHERE_WEIGHT = 2.5
CONDITION_WEIGHT = 3

SELECT_FIELD_WEIGHT = 1
SELECT_FROM_WEIGHT = 2
SELECT_TABLE_WEIGHT = 2.2

SELECT_JOIN_WEIGHT = 2.3
SELECT_JOINED_TABLE_WEIGHT = 2.4
SELECT_JOIN_CONDITION_WEIGHT = 2.5

SELECT_GROUP_WEIGHT = 2.6
SELECT_GROUP_FIELD = 2.7

SELECT_HAVING_WEIGHT = 2.8
SELECT_HAVING_CONDITION = 2.9

SELECT_ORDER_WEIGHT = 4
SELECT_ORDER_FIELD_WEIGHT = 5
SELECT_ORDER_ASC_WEIGHT = 6

SELECT_LIMIT_WEIGHT = 7
SELECT_LIMIT_VALUE_WEIGHT = 8

UPDATE_TABLE_WEIGHT = 1
UPDATE_SET_WEIGHT = 1.5
UPDATE_SET_FIELD_WEIGHT = 2

INSERT_TABLE_WEIGHT = 1
INSERT_VALUES_WEIGHT = 2

DELETE_FROM_WEIGHT = 1
DELETE_TABLE_WEIGHT = 2

CREATE_TABLE_WEIGHT = 1
CREATE_TABLE_FIELD_WEIGHT = 2
CREATE_TABLE_FIELDS_WEIGHT = 3
